<?php

namespace App\Repository;

use App\Models\Danger;
use App\Models\DangerLevelEvaluation;
use App\Models\Evaluation;
use App\Models\EvaluationMaturityAnswer;
use App\Models\Measure;
use App\Models\MeasureLevel;
use App\Models\Scenario;

class GraphDataRepository
{
    private EvaluationRepository $repository;

    public function __construct(EvaluationRepository $repository)
    {
        $this->repository = $repository;
    }

    public function getMeasuresDataForEvaluation(Evaluation $evaluation): array
    {
        $measureLevels = collect($evaluation->measureLevels)->sortBy(function (MeasureLevel $ml) {
            return $ml->measure->fundamental;
        });

        $levels = $measureLevels->map(function (MeasureLevel $level) {
            return $level->actual_level;
        });

        $labels = $measureLevels->map(function (MeasureLevel $level) {
            return $level->measure->short_name;
        });

        return [
            'labels' => $labels->values(),
            'data' => $levels->values(),
        ];
    }

    /**
     * For each danger, return the exposition value taking into account the danger level that was selected.
     */
    public function getExpositionDataForEvaluation(Evaluation $evaluation): array
    {
        $measureLevels = collect($evaluation->measureLevels);
        $dangerLevels = collect($evaluation->dangerLevels);

        // For each danger, add all points of all measures that apply to the danger scenarios

        $dangers = Danger::all();

        $labels = [];
        $values = [];

        foreach ($dangers as $danger) {
            /*
             * @var Danger $danger
             */
            $labels[] = $danger->name;

            $v = $danger->scenarios->reduce(function ($acc, Scenario $scenario) use ($measureLevels) {
                $exposition = $this->getExpositionPointsForScenario($scenario, $measureLevels);

                return $acc + $exposition;
            }, 0);

            // Get selected danger level
            $dangerLevel = $dangerLevels->first(function (DangerLevelEvaluation $dl) use ($danger) {
                return $dl->danger_id === $danger->id;
            });

            // the points for this danger are the total points for this danger,
            // divided by the number of appllicable scenarios
            $values[] = $v / $danger->scenarios->count() * $dangerLevel->level->getPointsAttribute();
        }

        $total = collect($values)->sum();

        $percentages = collect($values)->map(function ($val) use ($total) {
            if (0.0 === $val || 0.0 === $total) {
                return 10;
            }

            return max(10, $val / $total * 100);
        });

        return [
            'labels' => $labels,
            'data' => $percentages,
        ];
    }

    public function getRisksDataForEvaluation(Evaluation $evaluation): array
    {
        $measureLevels = collect($evaluation->measureLevels);

        // For each danger, add all points of all measures that apply to the danger scenarios

        $dangers = Danger::all();

        $labels = [];
        $values = [];

        foreach ($dangers as $danger) {
            /*
             * @var Danger $danger
             */
            $labels[] = $danger->name;

            $v = $danger->scenarios->reduce(function ($acc, Scenario $scenario) use ($measureLevels) {
                $exposition = $this->getExpositionPointsForScenario($scenario, $measureLevels);

                return $acc + $exposition;
            }, 0);

            $values[] = max(10, $v / $danger->scenarios->count());
        }

        return [
            'labels' => $labels,
            'data' => $values,
        ];
    }

    public function getFutureRisksDataForEvaluation(Evaluation $evaluation): array
    {
        $measureLevels = collect($evaluation->measureLevels);

        // For each danger, add all points of all measures that apply to the danger scenarios

        $dangers = Danger::all();

        $labels = [];
        $values = [];

        foreach ($dangers as $danger) {
            /*
             * @var Danger $danger
             */
            $labels[] = $danger->name;

            $v = $danger->scenarios->reduce(function ($acc, Scenario $scenario) use ($measureLevels) {
                $exposition = $this->getFutureExpositionPointsForScenario($scenario, $measureLevels);

                return $acc + $exposition;
            }, 0);

            $values[] = max(10, $v / $danger->scenarios->count());
        }

        return [
            'labels' => $labels,
            'data' => $values,
        ];
    }

    public function getAttackDataForEvaluation(Evaluation $evaluation): array
    {
        $scenarios = Scenario::with('measures')->get();

        $measureLevels = collect($evaluation->measureLevels);

        // For each scenario, add all points of all selected measure levels and divide by number of measures
        // (fundamental measures count double)

        $values = $scenarios->map(function (Scenario $scenario) use ($measureLevels) {
            return max(10, $this->getExpositionPointsForScenario($scenario, $measureLevels));
        });

        return [
            'labels' => $scenarios->pluck('name'),
            'data' => $values,
        ];
    }

    public function getMaturityDataForEvaluation(Evaluation $evaluation): array
    {
        // Calculate automatic answer results
        $evaluation->saveAutomaticAnswers();

        $evaluation = Evaluation::find($evaluation->id);

        $maturityLevels = $evaluation->maturityLevels;

        $values = $maturityLevels->map(function (EvaluationMaturityAnswer $ml) {
            return ['level' => $ml->answer->level, 'label' => $ml->question->title];
        });

        return [
            'labels' => $values->pluck('label'),
            'data' => $values->pluck('level'),
        ];
    }

    private function getExpositionPointsForScenario(Scenario $scenario, $measureLevels): float
    {
        // Get prorated count of measures
        $count = $scenario->measures->reduce(function ($acc, Measure $measure) {
            if ($measure->fundamental) {
                return $acc + 2;
            }

            return $acc + 1;
        }, 0);
        // for each measure in this scenario, add points for the level that is selected
        $points = $scenario->measures->reduce(function ($acc, Measure $measure) use ($measureLevels) {
            $selectedLevel = $measureLevels->first(function (MeasureLevel $level) use ($measure) {
                return $measure->id === $level->measure_id;
            });

            return $acc + $measure->{'level' . $selectedLevel->actual_level . '_value'};
        }, 0);

        return floor(100 - ($points / $count));
    }

    private function getFutureExpositionPointsForScenario(Scenario $scenario, $measureLevels): float
    {
        // Get prorated count of measures
        $count = $scenario->measures->reduce(function ($acc, Measure $measure) {
            if ($measure->fundamental) {
                return $acc + 2;
            }

            return $acc + 1;
        }, 0);
        // for each measure in this scenario, add points for the level that is selected
        $points = $scenario->measures->reduce(function ($acc, Measure $measure) use ($measureLevels) {
            $selectedLevel = $measureLevels->first(function (MeasureLevel $level) use ($measure) {
                return $measure->id === $level->measure_id;
            });

            $val = $measure->{'level' . $selectedLevel->expected_level . '_value'};
            if (!$val) {
                $val = $measure->{'level' . $selectedLevel->actual_level . '_value'};
            }

            return $acc + $val;
        }, 0);

        return floor(100 - ($points / $count));
    }
}
